/// <reference types="node" />
import { ApolloServer } from '@apollo/server';
import type { KeyValueCache } from '@apollo/utils.keyvaluecache';
import express, { Express, NextFunction, Request, RequestHandler, Response } from 'express';
import { RateLimitExceededEventHandler } from 'express-rate-limit';
import { Server } from 'http';
import { WebSocketServer } from 'ws';
import { AccessInfo, AppDomain, AuthBasicVerify, AuthBearerVerify, AuthDirective } from './common';
export { express, Express, NextFunction, Request, RequestHandler, Response, RateLimitExceededEventHandler };
export declare const DEFAULT_RATE_LIMIT_WINDOW: number;
export declare const DEFAULT_RATE_LIMIT_MAX_REQ_PER_WINDOW = 100;
export declare type AppBaseOptions = {
    errorUrl?: string;
    identityUrl?: string;
    fnBearerVerify?: AuthBearerVerify;
    fnVerify?: AuthBasicVerify;
    disableDebugTrace?: boolean;
    [k: string]: any;
};
export declare type AppBaseInitParams = {
    serviceIdToken?: string;
    orgId?: number;
    appDomain?: AppDomain;
    productId?: number;
    origins?: string;
    rateLimitHandler?: RateLimitExceededEventHandler;
    multipartFormData?: boolean;
    enablePubSub?: boolean;
    options?: AppBaseOptions;
    apolloOptions?: {
        [k: string]: any;
    };
};
export interface MyContext {
    req: Request;
    ip: string;
    accessInfo: AccessInfo;
    cache: KeyValueCache;
}
export declare abstract class AppBase {
    readonly name: string;
    readonly version: string;
    protected app: Express;
    protected listener: Server;
    protected httpServer: Server;
    protected wsServer: WebSocketServer;
    protected apolloServer: ApolloServer<MyContext>;
    protected correlator: any;
    protected serviceIdToken: string;
    protected serviceOrgId: number;
    protected serviceAppDomain: AppDomain;
    protected serviceProductId: number;
    private _devMode;
    private _terminated;
    protected abstract shutdown(): Promise<void>;
    protected abstract get liveness(): boolean;
    protected abstract get readiness(): boolean;
    constructor(name?: string, version?: string);
    protected init(params?: AppBaseInitParams): Promise<this>;
    private setupBaseMiddleware;
    private setupBaseComponents;
    private setupAuthComponents;
    term(): Promise<void>;
    run(port?: number, hostname?: string): Promise<string>;
    protected handler(thisObject: any, func: RequestHandler, authDirective: AuthDirective | undefined, logging?: boolean, raw?: boolean): RequestHandler;
    protected fileHandler(cachePath: string, maxFileSize: number, fileFieldName?: string): RequestHandler;
    protected bootstrap(): void;
    /**
     * @api {get} /liveness Liveness health check
     * @apiVersion 1.0.0
     * @apiName onLiveness
     * @apiGroup HealthCheck
     * @apiPermission none
     *
     * @apiDescription Service liveness health check.
     *
     * @apiSuccessExample Success-Response:
     *     HTTP/1.1 200 OK
     *
     * @apiErrorExample Error-Response:
     *     HTTP/1.1 500 Service is offline
     */
    private onLiveness;
    /**
     * @api {get} /readiness Readiness health check
     * @apiVersion 1.0.0
     * @apiName onReadiness
     * @apiGroup HealthCheck
     * @apiPermission none
     *
     * @apiDescription Service readiness health check.
     *
     * @apiSuccessExample Success-Response:
     *     HTTP/1.1 200 OK
     *
     * @apiErrorExample Error-Response:
     *     HTTP/1.1 429 Service is not ready
     */
    private onReadiness;
    /**
     * @api {get} / Service information
     * @apiVersion 1.0.0
     * @apiName onRoot
     * @apiGroup HealthCheck
     * @apiPermission none
     *
     * @apiDescription Service information.
     *
     * @apiSuccess {String} message Service information
     *
     * @apiSuccessExample Success-Response:
     *     HTTP/1.1 200 OK
     *     {
     *       "message": "Welcome to the coolest API on earth - ServiceName:1.0.0"
     *     }
     */
    private onRoot;
    protected get devMode(): boolean;
    protected set devMode(value: boolean);
}
