"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModelMiddleware = exports.OmitUpdateInput = exports.OmitCreateInput = exports.RecordStatus = void 0;
const lodash_1 = __importDefault(require("lodash"));
const async_hooks_1 = require("async_hooks");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const model_middleware_role_ability_log_1 = require("./model-middleware-role-ability-log");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus = exports.RecordStatus || (exports.RecordStatus = {}));
exports.OmitCreateInput = { createdBy: 0, createdByName: '' };
exports.OmitUpdateInput = { updatedBy: 0, updatedByName: '' };
class BaseModelMiddleware {
    client;
    errorHandler;
    asyncLocalStorage;
    constructor(client, errorHandler) {
        this.asyncLocalStorage = new async_hooks_1.AsyncLocalStorage();
        this.errorHandler = errorHandler;
        client['$use'](async (params, next) => this.process(params, next));
        if (model_middleware_role_ability_log_1.BaseModelMiddlewareRoleAbilityLog.isEnabled) {
            client['$use'](async (params, next) => model_middleware_role_ability_log_1.BaseModelMiddlewareRoleAbilityLog.instance.process(params, next, this.asyncLocalStorage));
        }
        this.client = client;
    }
    async run(context, promise) {
        return this.asyncLocalStorage.run(context, async () => {
            return await promise();
        });
    }
    async process(params, next) {
        const start = (0, cm_libs_utils_1.timeNow)();
        const context = this.asyncLocalStorage.getStore();
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `Prepare ${params.model}.${params.action}: ${JSON.stringify(params, null, 2)}`);
        switch (params.action) {
            case 'findUnique':
                this.packFindUniqueArgs(params, context);
                break;
            case 'findMany':
                this.packFindManyArgs(params, context);
                break;
            case 'create':
                this.packCreateArgs(params, context);
                break;
            case 'createMany':
                this.packCreateManyArgs(params, context);
                break;
            case 'update':
                this.packUpdateArgs(params, context);
                break;
            case 'updateMany':
                this.packUpdateManyArgs(params, context);
                break;
            case 'upsert':
                this.packUpsertArgs(params, context);
                break;
            case 'delete':
                this.packDeleteArgs(params, context);
                break;
            case 'deleteMany':
                this.packDeleteManyArgs(params, context);
                break;
        }
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DETAIL, () => `Run ${params.model}.${params.action}: ${JSON.stringify(params, null, 2)}`);
        try {
            const result = await next(params);
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `Run ${params.model}.${params.action} affected ${affectedCount(params.action, result)} record(s) took ${(0, cm_libs_utils_1.timeElapsed)(start)} ms`);
            return result;
        }
        catch (error) {
            this.errorHandler ? this.errorHandler(error, params, context) : (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.DATABASE_ERROR, { params, context });
        }
    }
    packFindUniqueArgs(params, context) {
        this.restrictInclude(params.args, params.action, params.model);
        if (!context.disableRestriction) {
            const roleModel = this.restrictSelect(params.args, params.action, params.model, context);
            this.restrictWhereUnique(params.args, roleModel?.[params.action]?.where);
        }
    }
    packFindManyArgs(params, context) {
        this.restrictInclude(params.args, params.action, params.model);
        if (!context.disableRestriction) {
            const roleModel = this.restrictSelect(params.args, params.action, params.model, context);
            this.restrictWhere(params.args, roleModel?.[params.action]?.where);
        }
    }
    packCreateArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.getRoleModel(params.action, params.model, context.accessInfo);
        const schema = context.schemas?.schemaCreateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.data = {
            ...validatedValues(params.args.data, schema),
            createdBy: requestedBy,
            createdByName: requestedByName
        };
        this.restrictCreate(params.args.data, roleModel, context);
    }
    packCreateManyArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.getRoleModel(params.action, params.model, context.accessInfo);
        const schema = context.schemas?.schemaCreateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...validatedValues(params.args.data[i], schema),
                createdBy: requestedBy,
                createdByName: requestedByName
            };
            this.restrictCreate(params.args.data[i], roleModel, context);
        }
    }
    packUpdateArgs(params, context) {
        if (!context.disableRestriction) {
            const roleModel = this.getRoleModel(params.action, params.model, context.accessInfo);
            this.restrictWhereUnique(params.args, roleModel?.[params.action]?.where);
        }
        const schema = context.schemas?.schemaUpdateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.data = {
            ...validatedValues(params.args.data, schema),
            updatedBy: requestedBy,
            updatedByName: requestedByName
        };
    }
    packUpdateManyArgs(params, context) {
        if (!context.disableRestriction) {
            const roleModel = this.getRoleModel(params.action, params.model, context.accessInfo);
            this.restrictWhere(params.args, roleModel?.[params.action]?.where);
        }
        const schema = context.schemas?.schemaUpdateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...validatedValues(params.args.data[i], schema),
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
        }
    }
    packUpsertArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.getRoleModel('update', params.model, context.accessInfo);
        this.restrictWhereUnique(params.args, roleModel?.update?.where);
        const schemaCreateInput = context.schemas?.schemaCreateInput;
        const schemaUpdateInput = context.schemas?.schemaUpdateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.create = {
            ...validatedValues(params.args.create, schemaCreateInput),
            createdBy: requestedBy,
            createdByName: requestedByName
        };
        this.restrictCreate(params.args.create, roleModel, context);
        params.args.update = {
            ...validatedValues(params.args.update, schemaUpdateInput),
            updatedBy: requestedBy,
            updatedByName: requestedByName
        };
    }
    packDeleteArgs(params, context) {
        if (!context.disableRestriction) {
            const roleModel = this.getRoleModel(params.action, params.model, context.accessInfo);
            this.restrictWhereUnique(params.args, roleModel?.[params.action]?.where);
        }
    }
    packDeleteManyArgs(params, context) {
        if (!context.disableRestriction) {
            const roleModel = this.getRoleModel(params.action, params.model, context.accessInfo);
            this.restrictWhere(params.args, roleModel?.[params.action]?.where);
        }
    }
    restrictCreate(data, roleModel, context) {
        if (data) {
            for (const key of Object.keys(data)) {
                const item = data[key];
                if ('object' === typeof item) {
                    const subItem = item.connectOrCreate || item;
                    if (undefined !== subItem?.create) {
                        if (Array.isArray(subItem.create)) {
                            subItem.create = subItem.create.map((value) => {
                                return { ...value, ...exports.OmitCreateInput };
                            });
                        }
                        else {
                            subItem.create = { ...subItem.create, ...exports.OmitCreateInput };
                        }
                        if (roleModel) {
                            const subItemModel = this.selectableModel(key, roleModel);
                            const subRoleModel = this.getRoleModel('create', subItemModel.model, context.accessInfo);
                            this.restrictFields(subItem.create, subRoleModel?.create.data);
                            if (item.connectOrCreate) {
                                const subRoleModel = this.getRoleModel('findUnique', subItemModel.model, context.accessInfo);
                                this.restrictWhereUnique(subItem, subRoleModel?.findUnique?.where);
                            }
                        }
                    }
                }
            }
            this.restrictFields(data, roleModel?.create?.data);
        }
    }
    restrictSelect(args, action, model, context) {
        const roleModel = this.getRoleModel(action, model, context.accessInfo);
        if (args.select && roleModel) {
            for (const key of Object.keys(args.select)) {
                const item = args.select[key];
                if ('object' === typeof item && item.select) {
                    const itemModel = this.selectableModel(key, roleModel);
                    const itemRoleModel = this.restrictSelect(item, action, itemModel.model, context);
                    if (!itemModel.oneToOne) {
                        this.restrictWhere(item, itemRoleModel?.findMany?.where);
                    }
                }
            }
        }
        return roleModel;
    }
    restrictInclude(args, action, model) {
        if (args.include) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model via include method', { action, model });
        }
    }
    restrictWhere(args, where) {
        if (where) {
            args.where ??= {};
            args.where.AND ??= [];
            if (Array.isArray(where)) {
                args.where.AND = args.where.AND.concat(where);
            }
            else {
                args.where.AND.push(where);
            }
        }
    }
    restrictWhereUnique(args, where) {
        if (where) {
            args.where ??= {};
            for (const key of Object.keys(where)) {
                if (undefined !== args.where[key] && !lodash_1.default.isEqual(args.where[key], where[key])) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model on restricted record', { field: key, expectedValue: where[key] });
                }
            }
            args.where = { ...args.where, ...where };
        }
    }
    restrictFields(fields, data) {
        if (data) {
            const objects = Array.isArray(fields) ? fields : [fields];
            for (const object of objects) {
                for (const key of Object.keys(object)) {
                    if (undefined !== data[key] && !lodash_1.default.isEqual(fields[key], data[key])) {
                        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model on restricted field', { field: key, expectedValue: data[key] });
                    }
                }
            }
        }
    }
    getRoleModel(action, modelName, accessInfo) {
        if (Array.isArray(accessInfo.auth?.rolesMetadata)) {
            let restriction = true;
            for (const roleMetadata of accessInfo.auth.rolesMetadata) {
                // Disable restriction, but give priority to defined restriction if any
                if (roleMetadata.models['*']) {
                    restriction = false;
                }
                const model = roleMetadata.models[modelName];
                if (model) {
                    if (action) {
                        if (model[action]) {
                            return model;
                        }
                        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to execute model', {
                            action,
                            model: modelName,
                            rolesMetadata: accessInfo.auth.rolesMetadata,
                            userId: accessInfo.requestedBy
                        });
                    }
                    return model;
                }
            }
            if (!restriction) {
                return undefined;
            }
        }
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model', {
            action,
            model: modelName,
            rolesMetadata: accessInfo.auth?.rolesMetadata,
            userId: accessInfo.requestedBy
        });
    }
    selectableModel(model, roleModel) {
        const selected = roleModel.select?.[model];
        if (undefined !== selected && 'object' === typeof selected) {
            return selected;
        }
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access foreign model', { model });
    }
}
exports.BaseModelMiddleware = BaseModelMiddleware;
function affectedCount(action, result) {
    let count;
    switch (action) {
        case 'findUnique':
        case 'update':
            count = result ? 1 : 0;
            break;
        case 'findMany':
        case 'queryRaw':
            count = result?.length;
            break;
        case 'count':
        case 'executeRaw':
            count = Number(result);
            break;
        case 'create':
        case 'upsert':
            count = 1;
            break;
        case 'createMany':
        case 'updateMany':
            count = result?.count;
    }
    return count || 0;
}
function validatedValues(data, schema) {
    const { error, value } = schema ? schema.validate(data, { stripUnknown: true }) : { error: undefined, value: data };
    if (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.VALIDATION_ERROR, { data });
    }
    return value;
}
//# sourceMappingURL=model-middleware.js.map